import logging
import os
import requests
from flask import Flask, abort, request
from flask.logging import create_logger
from prometheus_flask_exporter import PrometheusMetrics
import toml

from .j2_template_engine import load_j2_template_engine
from . import __version__

app = Flask(__name__)

# Load configuration file.
CONFIG = toml.load(os.environ.get('CONFIG_FILE_LOCATION', 'config.toml'))

# Logger Setup
LOGGER = create_logger(app)
LOGGER.setLevel(logging.INFO)

# Metrics set-up
metrics = PrometheusMetrics(app)
metrics.info('up', 'Application health')
metrics.info('info', 'Application information', version=__version__)

# Template Setup.
if 'custom_template_path' in CONFIG['app']['notification']:
    J2_TEMPLATE_ENGINE = load_j2_template_engine(CONFIG['app']['notification']['custom_template_path'])
else:
    dir_path = os.path.dirname(os.path.realpath(__file__))
    J2_TEMPLATE_ENGINE = load_j2_template_engine(dir_path + '/default-notification-template.json.j2')


@app.route('/healthz')
@metrics.do_not_track()
def healthz():
    """ Healthz endpoint """
    return ''


@app.route('/alerts', methods=['POST'])
def post_alerts():
    """ Parses the given message and post it to the indicated channel. """

    # Get room name and validate.
    room_name = request.args.get('room', '')
    if room_name == '':
        LOGGER.debug('Room <%s> not given as parameter.')
        abort(400)

    # Check if room is defined in configuration.
    if room_name not in CONFIG['app']['room']:
        LOGGER.debug('Room <%s> not present in configuration file.')
        abort(404)

    # Notification URL.
    notification_url = CONFIG['app']['room'][room_name]['notification_url']
    if 'origin' in CONFIG['app']['notification']:
        origin = CONFIG['app']['notification']['origin']
    else:
        origin = os.environ.get('HOSTNAME', 'Unknown')

    # Build a HTTP client request
    for alert in request.json['alerts']:
        # Render alert as text.
        render_payload = {
            'origin': origin
        }
        render_payload.update(alert)
        text_alert = J2_TEMPLATE_ENGINE.render(render_payload)

        # Post request.
        post_request_details = requests.post(
            notification_url,
            json={
                'text': text_alert
            },
            verify=False)

        if post_request_details.ok:
            LOGGER.info('Alert message posted successfully to GChat room <%s>.', room_name)
        else:
            LOGGER.error('Alert message failed to be posted to GChat room <%s>.', room_name)
            LOGGER.error('%s - %s',
                         str(post_request_details.status_code), str(post_request_details.text)
                         )

    return ''



from pathlib import Path

from jinja2 import Environment, FileSystemLoader


# Engine setup.
def load_j2_template_engine(template_file_path: str):
    """ Loads a J2 template engine from given template file path. """
    # Template file
    template_file = Path(template_file_path)

    # Template directory.
    resolved_template_dir_path = template_file.parent.resolve(strict=True)

    # Load J2 environment.
    j2_environment = Environment(
        loader=FileSystemLoader(str(resolved_template_dir_path))
    )

    # Return template.
    return j2_environment.get_template(template_file.name)



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="alertmanager-gchat-integration",
    version="1.0.0",
    description="The application starts a Web server which logs details of all incoming HTTP requests.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/julb/alertmanager-gchat-integration",
    author="Julb",
    author_email="julien@julb.me",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["alertmanager_gchat_integration"],
    include_package_data=True,
    install_requires=["flask==1.1.2", "prometheus-flask-exporter==0.18.1", "gunicorn==20.0.4", "toml", "requests", "jinja2"]
)
